/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.datacage.templating;

import java.util.List;
import java.util.UUID;

import javax.xml.xpath.XPathFunction;
import javax.xml.xpath.XPathFunctionException;

import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.AbstractFixBunduArtifact;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.access.FixRealizingAccess;
import org.dive4elements.river.artifacts.model.fixings.FixRealizingCalculationExtended;
import org.dive4elements.river.utils.RiverUtils;

/**
 * Very specialized function the resolves the 'year' (als middle of all used events) from a fix-analysis-vollmer
 * artifact).
 *
 * @author Gernot Belger
 */
public class FixAnalysisYearXPathFunction implements XPathFunction {

    public static final String ID = "fix_analysis_year";

    public static final int ARITY = 1;

    private final CallContext context;

    public FixAnalysisYearXPathFunction(final CallContext context) {
        this.context = context;
    }

    @Override
    public Object evaluate(final List args) throws XPathFunctionException {

        final UUID uuid = (UUID) args.get(0);
        if (uuid == null)
            return null;

        final D4EArtifact artifact = RiverUtils.getArtifact(uuid.toString(), this.context);
        if (!(artifact instanceof AbstractFixBunduArtifact))
            return null;

        final AbstractFixBunduArtifact fixation = (AbstractFixBunduArtifact) artifact;

        // Get W/Q input per gauge for this case.
        final FixRealizingAccess fixAccess = new FixRealizingAccess(fixation);

        /* ugly but necessary to keep this logic at least a bit inside the FixRealizing stuff */
        final FixRealizingCalculationExtended calculation = new FixRealizingCalculationExtended(fixAccess);
        return calculation.determineMeanYear();
    }
}